package com.sd.business.entity.order.po;

import com.baomidou.mybatisplus.annotation.TableName;
import com.ruoyi.common.core.domain.BasePo;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 订单
 * </p>
 *
 * @author
 * @since 2023-07-19
 */
@Getter
@Setter
@TableName("order_info")
public class OrderInfo extends BasePo {

    /**
     * 来源 1自定义添加 2万里牛
     */
    private Integer source;

    /**
     * 订单类型 1自主订单 2委外订单
     */
    @NotNull(message = "订单类型不能为空")
    private Integer type;

    /**
     * 事业部id
     */
    @NotNull(message = "事业部id不能为空")
    private Long departmentId;

    /**
     * 订单号
     */
    private String code;

    /**
     * 万里牛uid
     */
    private String wlnUid;

    /**
     * 万里牛订单号
     */
    private String wlnCode;

    /**
     * 万里牛订单状态
     * -3:分销商审核
     * -2:到账管理
     * -1:未付款
     * 0:审核
     * 1:打单配货
     * 2:验货
     * 3:称重
     * 4:待发货
     * 5：财审
     * 8:已发货
     * 9:成功
     * 10:关闭
     * 11:异常结束
     * 12:异常处理
     * 13:外部系统配货中
     * 14:预售
     * 15:打包
     * 19:拣货
     */
    private Integer wlnStatus;

    /**
     * 万里牛订单创建时间
     */
    private Date wlnCreateTime;

    /**
     * 万里牛订单审核时间
     */
    private Date wlnApproveTime;

    /**
     * 万里牛打单时间
     */
    private Date wlnPrintTime;

    /**
     * 万里牛订单修改时间
     */
    private Date wlnModifyTime;

    /**
     * 万里牛同步时间戳
     */
    private Long wlnModifyTimestamp;

    /**
     * 万里牛仓库编码
     */
    private String wlnStorageCode;

    /**
     * 交期
     */
    @NotNull(message = "交货时间不能为空")
    private Date deliveryTime;

    /**
     * 店铺来源
     */
    private String sourcePlatform;

    /**
     * 店铺名称
     */
    private String shopName;

    /**
     * 进度 {@link com.sd.business.entity.order.enums.OrderStatusEnum}
     */
    @NotNull(message = "订单进度不能为空")
    private Integer status;

    /**
     * 异常类型 {@link com.sd.business.entity.order.enums.OrderExceptionTypeEnum}
     */
    private String exceptionType;

    /**
     * 结算状态 1未结算 2已对账 3已结算
     */
    private Integer settlementStatus;

    /**
     * 订单总金额
     */
    @NotNull(message = "订单总金额不能为空")
    private BigDecimal totalAmount;

    /**
     * 产品总金额
     */
    @NotNull(message = "产品总金额不能为空")
    private BigDecimal productTotalAmount;

    /**
     * 定制加工费
     */
    @NotNull(message = "定制加工费不能为空")
    private BigDecimal customProcessingFee;

    /**
     * 代发费
     */
    @NotNull(message = "代发费不能为空")
    private BigDecimal lssueFee;

    /**
     * 快递包材费
     */
    @NotNull(message = "快递包材费不能为空")
    private BigDecimal deliveryMaterialsFee;

    /**
     * 包装人工费
     */
    @NotNull(message = "包装人工费不能为空")
    private BigDecimal packingLabor;

    /**
     * 管理费
     */
    @NotNull(message = "管理费不能为空")
    private BigDecimal managementFee;

    /**
     * 包材费
     */
    @NotNull(message = "包材费不能为空")
    private BigDecimal packagingMaterialCost;

    /**
     * 税率
     */
    private BigDecimal taxRate;

    /**
     * 发货时间
     */
    private Date shippingTime;

    /**
     * 收货人
     */
    @NotBlank(message = "联系人不能为空")
    private String consignee;

    /**
     * 收货人电话
     */
    @NotBlank(message = "联系电话不能为空")
    private String consigneeNumber;

    /**
     * 省
     */
    @NotBlank(message = "省不能为空")
    private String province;

    /**
     * 市
     */
    private String city;

    /**
     * 县
     */
    private String county;

    /**
     * 详细地址
     */
    @NotBlank(message = "详细地址不能为空")
    private String detailedAddress;

    /**
     * 邮编
     */
    @NotBlank(message = "邮编不能为空")
    private String postcode;

    /**
     * 快递网点id
     */
    private String expressDeliveryId;

    /**
     * 快递单号
     */
    private String expressDeliveryCode;

    /**
     * 对账单id
     */
    private Long statementOfAccountId;

    /**
     * 标签
     */
    private String tag;

    /**
     * 出库事业部
     */
    private Long outDepartmentId;

    /**
     * 流程id
     */
    private Long flowId;

    /**
     * 流程状态 0未发起 1进行中 2已通过 3已驳回 4已作废
     */
    private Integer flowStatus;

    /**
     * 备注
     */
    private String remark;

    /**
     * 订单删除标记
     */
    private Integer delFlag;

}
