package com.sd.business.mapper.inventory;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.in.po.InOutStorageDetails;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;


/**
 * <p>
 * 库存 Mapper 接口
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
public interface InventoryMapper extends BaseMapper<Inventory> {

    /**
     * 库存分页
     */
    Page<InventoryVo> getPage(@Param("page") Page<Object> page,
                              @Param("ew") IWrapper<Inventory> wrapper,
                              @Param("tableName") String tableName);

    /**
     * 根据仓库获取库存数量
     */
    List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto);

    /**
     * 根据事业部获取库存数量
     */
    List<QuantityByDepartmentVo> getQuantityByDepartment();

    /**
     * 获取出入库详情表的数据
     */
    List<InOutStorageDetails> getInOutDetails(@Param("warehouseId") Long warehouseId,
                                              @Param("departmentId") Long departmentId,
                                              @Param("bomSpecId") Long bomSpecId);

    /**
     * 根据bomSpecId获取结存数量
     */
    BigDecimal getQuantityByBomSpecId(@Param("bomSpecId") Long bomSpecId, @Param("warehouseId") Long warehouseId);

    /**
     * 库存列表
     */
    List<InventoryVo> getList(@Param("ew") IWrapper<Inventory> wrapper);

}
