package com.sd.business.service.outbound.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.dto.OrderSelectDto;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.outbound.dto.OutboundOrderDto;
import com.sd.business.entity.outbound.dto.OutboundOrderSelectDto;
import com.sd.business.entity.outbound.po.OutboundOrder;
import com.sd.business.entity.outbound.vo.OutboundOrderVo;
import com.sd.business.mapper.outbound.OutboundOrderMapper;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.outbound.OutboundOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 出库单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-27
 */
@Service
public class OutboundOrderServiceImpl extends ServiceImpl<OutboundOrderMapper, OutboundOrder> implements OutboundOrderService {

    @Autowired
    private OrderService orderService;

    @Override
    public Page<OutboundOrderVo> getPage(OutboundOrderSelectDto dto) {
        IWrapper<OutboundOrder> wrapper = getWrapper();
        wrapper.orderByDesc("oo", OutboundOrder::getId);
        Page<OutboundOrderVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public OutboundOrderVo detail(Long id) {
        OutboundOrder OutboundOrder = this.getById(id);
        OutboundOrderVo result = BeanUtil.toBean(OutboundOrder, OutboundOrderVo.class);
        return result;
    }

    @Override
    public void add(OutboundOrderDto outboundOrderDto) {
        this.save(outboundOrderDto);
    }

    @Override
    public void edit(OutboundOrderDto outboundOrderDto) {
        this.updateById(outboundOrderDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public List<String> orderDeliveryComparison(OrderSelectDto dto) {
        dto.setPageNum(1);
        dto.setPageSize(999999999);
        Page<OrderInfoVo> page = orderService.getPage(dto);
        List<OrderInfoVo> records = page.getRecords();
        if (records.size() == 0) {
            return Collections.emptyList();
        }

        List<String> wlnCodeList = records.stream().map(OrderInfo::getWlnCode).collect(Collectors.toList());
        List<OutboundOrder> list = list(q -> q.in(OutboundOrder::getOrderWlnCode, wlnCodeList));
        List<String> outboundOrderWlnCodeList = list.stream().map(OutboundOrder::getOrderWlnCode).collect(Collectors.toList());

        return wlnCodeList.stream().filter(item -> !outboundOrderWlnCodeList.contains(item)).collect(Collectors.toList());
    }

}
