import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.DynamicRoutingDataSource;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import com.sd.SdApplication;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.test.context.junit4.SpringRunner;

import javax.sql.DataSource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 同步用户表
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = SdApplication.class, webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
public class A9_SyncUserTest {

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private DataSource dataSource;

    @DSTransactional
    @Test
    public void test() {
        DynamicRoutingDataSource dynamicRoutingDataSource = (DynamicRoutingDataSource) dataSource;
        DataSource sdDataSource = dynamicRoutingDataSource.getDataSource("shengde");



        String sql2 = "SELECT\n" +
                "\t id oldId,\n" +
                "\t role_name roleName,\n" +
                "\t role_alias roleKey,\n" +
                "\t sort roleSort,\n" +
                "\t tenant_id tenantId\n" +
                "\t\n" +
                "FROM\n" +
                "\t blade_role" +
                "\t WHERE role_alias != 'administrator'";
        List<Map<String, Object>> roleList = new JdbcTemplate(sdDataSource).queryForList(sql2);

        List<SysRole> sysRoleList = BeanUtil.copyToList(roleList, SysRole.class);
        for (SysRole sysRole : sysRoleList) {
            sysRoleService.insertRole(sysRole);
        }

        String sql = "SELECT\n" +
                "\t role_id roleId,\n" +
                "\t account userName,\n" +
                "\t name nickName,\n" +
                "\t email,\n" +
                "\t phone phonenumber,\n" +
                "\t (CASE sex WHEN 1 THEN '0' WHEN 0 THEN '1' ELSE '2' END) sex,\n" +
                "\t avatar,\n" +
                "\t create_time createTime,\n" +
                "\t update_time updateTime,\n" +
                "\t '000000' tenantId,\n" +
                "\t code jobNumber,\n" +
                "\t dm_code userCode\n" +
                "\t\n" +
                "FROM\n" +
                "\t blade_user";

        List<Map<String, Object>> list = new JdbcTemplate(sdDataSource).queryForList(sql);
        List<Map<String, Object>> userList = list.stream().filter(item -> {
            String userName = item.get("userName").toString();
            SysUser sysUser = sysUserService.selectUserByUserName("000000", userName);
            return ObjectUtil.isEmpty(sysUser);
        }).peek(item -> {
            String password = "123456";
            BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
            String encryptPassword = passwordEncoder.encode(password);
            item.put("password", encryptPassword);
        }).collect(Collectors.toList());
        List<SysUser> sysUserList = BeanUtil.copyToList(userList, SysUser.class);
        for (SysUser sysUser : sysUserList) {
            sysUserService.save(sysUser);
        }

        // 用户角色绑定
        for (Map<String, Object> map : userList) {
            String userName = map.get("userName").toString();
            String tenantId = map.get("tenantId").toString();
            String roleId = map.get("roleId").toString();
            SysUser sysUser = sysUserService.selectUserByUserName(tenantId, userName);
            List<String> collect = Arrays.asList(roleId.split(","));
            List<String> roleKeys = roleList.stream().filter(item -> collect.contains(item.get("oldId").toString())).map(item -> item.get("roleKey").toString()).collect(Collectors.toList());
            List<Long> roleIdList = sysRoleList.stream().filter(item -> roleKeys.contains(item.getRoleKey())).map(SysRole::getRoleId).collect(Collectors.toList());
            sysUserService.insertUserAuth(sysUser.getUserId(),  roleIdList.toArray(new Long[0]));
        }
    }
}
