import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.read.builder.ExcelReaderBuilder;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.sd.SdApplication;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.excel.listener.DataListener;
import lombok.Getter;
import lombok.Setter;
import lombok.SneakyThrows;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.FileInputStream;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 同步sku图稿库
 */
@RunWith(SpringRunner.class)
@SpringBootTest(classes = SdApplication.class, webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
public class B1_SyncSkuTest {


    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @SneakyThrows
    @DSTransactional
    @Test
    public void test() {

        DataListener<ExcelData> listener = new DataListener<>();

        FileInputStream fileInputStream = new FileInputStream("E:\\峰燕-胜德体育-品号汇总模板-2 (2).xlsx");

        ExcelReaderBuilder read = EasyExcel.read(fileInputStream, ExcelData.class, listener);
        read.sheet(0).doRead();
        List<ExcelData> list = listener.getDataList();
        Map<String, ExcelData> map = list.stream().collect(Collectors.toMap(ExcelData::getCode, Function.identity()));

        List<SkuSpec> skuSpecList = skuSpecService.list();
        List<ArtworkLibrary> artworkLibraryList = artworkLibraryService.list();
        Map<String, ArtworkLibrary> artworkLibraryMap = artworkLibraryList.stream().collect(Collectors.toMap(ArtworkLibrary::getArtworkName, Function.identity()));

        // 机关图案算体位线
        for (SkuSpec skuSpec : skuSpecList) {
            String code = skuSpec.getCode();
            ExcelData excelData = map.get(code);

            if (excelData == null) {
                if (code.startsWith("ST") || code.startsWith("SK")) {
                    String name = skuSpec.getName();

                    if (code.endsWith("T")) {
                        if (name.contains("183*80")) {
                            ArtworkLibrary artworkLibrary = artworkLibraryMap.get("体位线-183_80");
                            skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                            skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                            System.out.println(80);
                        } else if (name.contains("183*68")) {
                            ArtworkLibrary artworkLibrary = artworkLibraryMap.get("体位线-183_68");
                            skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                            skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                            System.out.println(68);
                        } else if (name.contains("183*66")) {
                            ArtworkLibrary artworkLibrary = artworkLibraryMap.get("体位线_183_66");
                            skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                            skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                            System.out.println(66);
                        } else if (name.contains("183*61")) {
                            ArtworkLibrary artworkLibrary = artworkLibraryMap.get("体位线_183_61");
                            skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                            skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                            System.out.println(61);
                        }
                        skuSpec.setMachinedPanel("10");
                    }

                    if (code.endsWith("W")) {
                        ArtworkLibrary artworkLibrary = artworkLibraryMap.get("EMPTY");
                        skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                        skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                    }

                    if (code.endsWith("CG")) {
                        ArtworkLibrary artworkLibrary = artworkLibraryMap.get("跳绳垫165_61");
                        skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                        skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                        skuSpec.setMachinedPanel("10");
                    }

                    if (code.endsWith("DT")) {
                        skuSpec.setDesignImgUrl("");
                        skuSpec.setSharedFolder("");
                        skuSpec.setMachinedPanel("10");
                    }

                    if (code.endsWith("L")) {
                        skuSpec.setDesignImgUrl("");
                        skuSpec.setSharedFolder("");
                        skuSpec.setMachinedPanel("20");
                    }

                }
                continue;
            }

            String machinedPanelStr = excelData.getMachinedPanelStr();
            if (StrUtil.isNotBlank(machinedPanelStr)) {
                machinedPanelStr = machinedPanelStr.replace(" ", "");
                if (machinedPanelStr.startsWith("激光体位线") || machinedPanelStr.startsWith("激光图案")) {
                    skuSpec.setMachinedPanel("10");
                } else if (machinedPanelStr.startsWith("激光logo")) {
                    skuSpec.setMachinedPanel("20");
                }
            }

            String machinedPanelFileStr = excelData.getMachinedPanelFileStr();
            if (StrUtil.isBlank(machinedPanelFileStr)) {
                ArtworkLibrary artworkLibrary = artworkLibraryMap.get("EMPTY");
                skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                continue;
            }


            String spec = excelData.getSpec();
            if (StrUtil.isBlank(spec)) {
                continue;
            }
            String sp = spec.split("\\*")[1].replace("cm", "").replace(" ", "");
            for (ArtworkLibrary artworkLibrary : artworkLibraryList) {
                String[] ar = artworkLibrary.getArtworkName().split("_");
                String ar0 = ar[0];
                String arl = ar[ar.length - 1];

                if (machinedPanelFileStr.startsWith(ar0) && arl.equals(sp)) {
                    skuSpec.setDesignImgUrl(artworkLibrary.getImgUrl());
                    skuSpec.setSharedFolder(artworkLibrary.getFileUrl());
                    break;
                }
            }

        }

        skuSpecService.updateBatchById(skuSpecList);

        System.out.println();
    }

    @Getter
    @Setter
    public static final class ExcelData {

        @ExcelProperty("SKU")
        private String code;

        @ExcelProperty("图稿类型")
        private String machinedPanelStr;

        @ExcelProperty("图稿文件")
        private String machinedPanelFileStr;

        @ExcelProperty("规格")
        private String spec;

    }

}
