package com.sd.wln.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.ruoyi.common.annotation.LogicIgnore;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.service.order.OrderService;
import com.sd.wln.context.OrderContext;
import com.sd.wln.entity.ResynchronizationDto;
import com.sd.wln.service.OrderHandleService;
import com.sd.wln.service.WlnOrderService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.Objects;

@Slf4j
@Service
public class OrderHandleServiceImpl implements OrderHandleService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private WlnOrderService wlnOrderService;

    @DSTransactional
    @LogicIgnore
    @Override
    public synchronized void resynchronization(ResynchronizationDto dto) {
        JSONObject wlnOrder;
        try {
            wlnOrder = WlnUtil.getOrderByWlnCode(dto.getWlnCode());
        } catch (ServiceException e) {
            throw e;
        } catch (Exception e) {
            log.error("订单同步失败", e);
            throw new ServiceException("订单同步失败");
        }

        String uid = wlnOrder.getString("uid");
        OrderInfo orderInfo = orderService.getOne(q -> q.eq(OrderInfo::getWlnUid, uid));
        if (orderInfo != null) {
            if (Objects.equals(orderInfo.getDelFlag(), StatusConstant.NOT_DELETED)) {
                throw new ServiceException("订单未被删除，无法重新同步");
            }
            orderService.delete(orderInfo.getId(), false);
        }

        String storageCode = wlnOrder.getString("storage_code");
        OrderContext context = new OrderContext(storageCode, Collections.singletonList(wlnOrder));
        OrderInfo order = wlnOrderService.createOrder(context, wlnOrder);
        wlnOrderService.addOrder(context, wlnOrder, order);
        wlnOrderService.saveOrUpdateOrder(context);
    }

}
