package com.sd.wln.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fjhx.tenant.entity.dict.po.DictCommonData;
import com.fjhx.tenant.service.dict.DictCommonDataService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.enums.OrderExceptionTypeEnum;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.wln.context.OrderContext;
import com.sd.wln.service.WlnOrderService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WlnOrderServiceImpl implements WlnOrderService {

    @Autowired
    private PlatformTransactionManager platformTransactionManager;

    @Autowired
    private TransactionDefinition transactionDefinition;

    @Autowired
    private DictCommonDataService dictCommonDataService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Override
    public void syncOrder() {

        LogicHolder.setLogicHolder(false);

        List<DictCommonData> warehouseCodeList = dictCommonDataService.list(q -> q.eq(DictCommonData::getDictCode, "warehouse_code"));
        if (warehouseCodeList.size() == 0) {
            log.error("订单同步失败，仓库字典:warehouse_code 为空");
            return;
        }

        for (DictCommonData dictCommonData : warehouseCodeList) {
            String warehouseCode = dictCommonData.getDictValue();
            List<JSONObject> wlnOrderList = getWlnOrderList(warehouseCode);

            // 万里牛订单若为0，结束同步任务
            if (ObjectUtil.isEmpty(wlnOrderList)) {
                continue;
            }

            OrderContext context = new OrderContext(warehouseCode, wlnOrderList);

            Map<String, OrderInfo> existOrderMap = context.getExistOrderMap();

            // 循环万里牛订单
            for (JSONObject wlnOrder : wlnOrderList) {

                Date printTime = wlnOrder.getDate("print_time");
                if (printTime != null && printTime.before(DateUtil.parse("2023-09-01 00:00:00"))) {
                    continue;
                }

                Integer processStatus = wlnOrder.getInteger("process_status");
                if (processStatus < 1) {
                    continue;
                }

                // 数据库中是否存在订单
                OrderInfo orderInfo = existOrderMap.get(wlnOrder.getString("uid"));

                // 不存在新增
                if (orderInfo == null) {
                    // 创建订单
                    orderInfo = createOrder(context, wlnOrder);
                    // 添加订单到上下文
                    addOrder(context, wlnOrder, orderInfo);
                }
                // 存在修改
                else {
                    updateOrder(context, wlnOrder, orderInfo);
                }

            }

            // 开启事务
            TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
            try {
                // 保存或更新数据
                saveOrUpdateOrder(context);
                // 提交事务
                platformTransactionManager.commit(transactionStatus);
            } catch (Exception e) {
                log.error("同步订单失败", e);
                // 回滚事务
                platformTransactionManager.rollback(transactionStatus);
            }

        }

        LogicHolder.clear();
    }

    /**
     * 创建订单
     */
    @Override
    public OrderInfo createOrder(OrderContext context, JSONObject wlnOrder) {

        OrderInfo orderInfo = new OrderInfo();
        orderInfo.setId(IdWorker.getId());
        orderInfo.setSource(2);
        orderInfo.setTaxRate(new BigDecimal("10.00"));
        orderInfo.setTag("0");
        orderInfo.setSettlementStatus(1);

        orderInfo.setWlnStorageCode(wlnOrder.getString("storage_code"));
        orderInfo.setWlnUid(wlnOrder.getString("uid"));
        orderInfo.setCode(wlnOrder.getString("tp_tid"));
        orderInfo.setWlnCode(wlnOrder.getString("trade_no"));
        orderInfo.setWlnStatus(wlnOrder.getInteger("process_status"));
        orderInfo.setWlnCreateTime(wlnOrder.getDate("create_time"));
        orderInfo.setWlnModifyTime(wlnOrder.getDate("modify_time"));
        orderInfo.setWlnModifyTimestamp(wlnOrder.getLong("modify_time"));
        orderInfo.setWlnApproveTime(wlnOrder.getDate("approve_time"));
        orderInfo.setWlnPrintTime(wlnOrder.getDate("print_time"));
        orderInfo.setSourcePlatform(wlnOrder.getString("source_platform"));
        orderInfo.setShopName(wlnOrder.getString("shop_name"));
        orderInfo.setProvince(wlnOrder.getString("province"));
        orderInfo.setCity(wlnOrder.getString("city"));
        orderInfo.setCounty(wlnOrder.getString("district"));
        orderInfo.setDetailedAddress(wlnOrder.getString("address"));
        orderInfo.setPostcode(wlnOrder.getString("zip"));
        orderInfo.setDeliveryTime(DateUtil.offsetDay(orderInfo.getWlnCreateTime(), 1));
        orderInfo.setRemark(wlnOrder.getString("remark"));

        orderInfo.setTotalAmount(BigDecimal.ZERO);
        orderInfo.setProductTotalAmount(BigDecimal.ZERO);
        orderInfo.setCustomProcessingFee(BigDecimal.ZERO);
        orderInfo.setLssueFee(BigDecimal.ZERO);
        orderInfo.setDeliveryMaterialsFee(BigDecimal.ZERO);
        orderInfo.setPackingLabor(BigDecimal.ZERO);
        orderInfo.setManagementFee(BigDecimal.ZERO);
        orderInfo.setPackagingMaterialCost(BigDecimal.ZERO);

        // 赋值订单异常类型
        orderInfo.setExceptionType(OrderExceptionTypeEnum.NORMAL.getKey().toString());
        if (orderInfo.getWlnStatus() == 10) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_CLOSURE);
        }
        if (orderInfo.getWlnStatus() == 11) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_ABORT);
        }
        if (orderInfo.getWlnStatus() == 12) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING);
        }

        // 事业部id
        Department department = context.getDepartment();
        if (department == null) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.UNKNOWN_WAREHOUSE);
        } else {
            orderInfo.setDepartmentId(department.getId());
        }

        // 订单进度
        if (department != null && Objects.equals(department.getOrderMode(), "2")) {
            orderInfo.setStatus(OrderStatusEnum.STOCK_PREPARATION.getKey());
        } else {
            orderInfo.setStatus(OrderStatusEnum.UNDER_REVIEW.getKey());
        }

        return orderInfo;
    }

    /**
     * 修改订单
     */
    private void updateOrder(OrderContext context, JSONObject wlnOrder, OrderInfo orderInfo) {
        if (orderInfo.getDelFlag().equals(StatusConstant.DELETED)) {
            return;
        }

        Department department = context.getDepartment();
        Map<Long, List<OrderSku>> existOrderSkuMap = context.getExistOrderSkuMap();
        List<OrderInfo> updateOrderList = context.getUpdateOrderList();

        orderInfo.setWlnStorageCode(wlnOrder.getString("storage_code"));
        orderInfo.setWlnUid(wlnOrder.getString("uid"));
        orderInfo.setCode(wlnOrder.getString("tp_tid"));
        orderInfo.setWlnCode(wlnOrder.getString("trade_no"));
        orderInfo.setWlnStatus(wlnOrder.getInteger("process_status"));
        orderInfo.setWlnCreateTime(wlnOrder.getDate("create_time"));
        orderInfo.setWlnModifyTime(wlnOrder.getDate("modify_time"));
        orderInfo.setWlnModifyTimestamp(wlnOrder.getLong("modify_time"));
        orderInfo.setWlnApproveTime(wlnOrder.getDate("approve_time"));
        orderInfo.setWlnPrintTime(wlnOrder.getDate("print_time"));
        orderInfo.setSourcePlatform(wlnOrder.getString("source_platform"));
        orderInfo.setShopName(wlnOrder.getString("shop_name"));
        orderInfo.setProvince(wlnOrder.getString("province"));
        orderInfo.setCity(wlnOrder.getString("city"));
        orderInfo.setCounty(wlnOrder.getString("district"));
        orderInfo.setDetailedAddress(wlnOrder.getString("address"));
        orderInfo.setPostcode(wlnOrder.getString("zip"));
        orderInfo.setDeliveryTime(DateUtil.offsetDay(orderInfo.getWlnCreateTime(), 1));
        orderInfo.setRemark(wlnOrder.getString("remark"));

        // 赋值订单异常类型
        if (orderInfo.getWlnStatus() == 10) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_CLOSURE);
        }
        if (orderInfo.getWlnStatus() == 11) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_ABORT);
        }
        if (orderInfo.getWlnStatus() == 12) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.ORDER_EXCEPTION_HANDLING);
        }

        // 事业部
        if (department == null) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.UNKNOWN_WAREHOUSE);
        } else {
            orderInfo.setDepartmentId(department.getId());
        }

        List<OrderSku> orderSkuList = new ArrayList<>(existOrderSkuMap.getOrDefault(orderInfo.getId(), Collections.emptyList()));
        List<JSONObject> wlnOrderSkuList = wlnOrder.getJSONArray("orders").toJavaList(JSONObject.class);

        if (orderSkuList.size() != wlnOrderSkuList.size()) {
            addExceptionType(orderInfo, OrderExceptionTypeEnum.SKU_UPDATE);
        } else {
            for (JSONObject wlnOrderSku : wlnOrderSkuList) {
                for (OrderSku orderSku : orderSkuList) {
                    if (Objects.equals(wlnOrderSku.getString("sys_goods_uid"), orderSku.getWlnSkuId())
                            && Objects.equals(wlnOrderSku.getString("sys_spec_uid"), orderSku.getWlnSkuSpecId())
                            && Objects.equals(wlnOrderSku.getString("oln_sku_name"), orderSku.getWlnSkuName())
                            && wlnOrderSku.getBigDecimal("size").compareTo(orderSku.getQuantity()) == 0) {
                        orderSkuList.remove(orderSku);
                        break;
                    }
                }
            }

            if (orderSkuList.size() != 0) {
                addExceptionType(orderInfo, OrderExceptionTypeEnum.SKU_UPDATE);
            }
        }

        updateOrderList.add(orderInfo);
    }

    /**
     * 创建订单包材
     */
    private List<OrderSkuBom> createOrderSkuBoom(OrderContext context, OrderSku orderSku) {
        Map<Long, Map<Integer, List<SkuSpecLink>>> linkBomSpecMap = context.getSkuSpecLinkBomSpecMap();
        Map<Long, BomSpecBo> bomBoMap = context.getBomSpecBoMap();

        Map<Integer, List<SkuSpecLink>> map = linkBomSpecMap.getOrDefault(orderSku.getSkuSpecId(), Collections.emptyMap());
        List<SkuSpecLink> skuSpecLinkList = map.getOrDefault(1, Collections.emptyList());

        return skuSpecLinkList.stream().map(item -> {
            OrderSkuBom orderSkuBom = new OrderSkuBom();
            orderSkuBom.setId(IdWorker.getId());
            orderSkuBom.setOrderId(orderSku.getOrderId());
            orderSkuBom.setOrderSkuId(orderSku.getId());
            orderSkuBom.setBomSpecId(item.getBomSpecId());
            orderSkuBom.setQuantity(item.getQuantity());
            BomSpecBo bomSpecBo = bomBoMap.get(orderSkuBom.getBomSpecId());
            if (bomSpecBo == null) {
                orderSkuBom.setUnitPrice(BigDecimal.ZERO);
            } else {
                orderSkuBom.setUnitPrice(bomSpecBo.getInternalSellingPrice());
                orderSku.setPackagingMaterialCost(orderSku.getPackagingMaterialCost()
                        .add(bomSpecBo.getInternalSellingPrice().multiply(item.getQuantity())));
            }
            return orderSkuBom;
        }).collect(Collectors.toList());

    }

    /**
     * 创建订单包装
     */
    private List<OrderPackageBom> createOrderPackageBomList(OrderContext context, List<OrderSku> orderSkuList) {
        Map<Long, Map<Integer, List<SkuSpecLink>>> linkBomSpecMap = context.getSkuSpecLinkBomSpecMap();
        Map<Long, BomSpecBo> bomBoMap = context.getBomSpecBoMap();

        Map<Long, OrderPackageBom> orderPackageBomMap = new HashMap<>();

        // 循环订单sku规格
        for (OrderSku orderSku : orderSkuList) {

            Map<Integer, List<SkuSpecLink>> map = linkBomSpecMap.getOrDefault(orderSku.getSkuSpecId(), Collections.emptyMap());
            List<SkuSpecLink> skuSpecLinkList = map.getOrDefault(2, Collections.emptyList());

            for (SkuSpecLink skuSpecLink : skuSpecLinkList) {
                Long bomSpecId = skuSpecLink.getBomSpecId();

                OrderPackageBom orderPackageBom = orderPackageBomMap.computeIfAbsent(bomSpecId, item -> {
                    OrderPackageBom tempOrderPackageBom = new OrderPackageBom();
                    tempOrderPackageBom.setOrderId(orderSku.getOrderId());
                    tempOrderPackageBom.setBomSpecId(bomSpecId);
                    tempOrderPackageBom.setQuantity(BigDecimal.ZERO);
                    BomSpecBo bomSpecBo = bomBoMap.get(bomSpecId);
                    if (bomSpecBo == null) {
                        tempOrderPackageBom.setCostPrice(BigDecimal.ZERO);
                        tempOrderPackageBom.setInternalSellingPrice(BigDecimal.ZERO);
                    } else {
                        tempOrderPackageBom.setCostPrice(bomSpecBo.getCostPrice());
                        tempOrderPackageBom.setInternalSellingPrice(bomSpecBo.getInternalSellingPrice());
                    }
                    return tempOrderPackageBom;
                });

                orderPackageBom.setQuantity(orderPackageBom.getQuantity().add(orderSku.getQuantity().multiply(skuSpecLink.getQuantity())));
            }
        }

        return new ArrayList<>(orderPackageBomMap.values());
    }

    /**
     * 添加订单到上下文
     */
    @Override
    public void addOrder(OrderContext context, JSONObject wlnOrder, OrderInfo orderInfo) {
        Map<String, SkuSpec> skuSpecMap = context.getSkuSpecMap();

        List<OrderSku> orderSkuList = new ArrayList<>();
        List<OrderSkuBom> orderSkuBomList = new ArrayList<>();

        // 创建订单sku
        for (JSONObject wlnOrderSku : wlnOrder.getJSONArray("orders").toJavaList(JSONObject.class)) {

            // 通过万里牛sku规格uid找到sku规格
            SkuSpec skuSpec = skuSpecMap.get(wlnOrderSku.getString("sys_spec_uid"));
            if (skuSpec == null) {
                addExceptionType(orderInfo, OrderExceptionTypeEnum.UNKNOWN_SKU_SPEC);
            } else if (skuSpec.getBomSpecId() == null) {
                addExceptionType(orderInfo, OrderExceptionTypeEnum.UNKNOWN_BOM_SPEC);
            }

            // 创建订单sku
            OrderSku orderSku = createOrderSku(context, wlnOrderSku, orderInfo, skuSpec);

            // 获取订单sku包材
            List<OrderSkuBom> tempOrderSkuBomList = createOrderSkuBoom(context, orderSku);

            // 添加订单金额
            addOrderInfoPrice(orderInfo, orderSku);

            orderSkuList.add(orderSku);
            orderSkuBomList.addAll(tempOrderSkuBomList);
        }

        List<OrderPackageBom> orderPackageBomList = createOrderPackageBomList(context, orderSkuList);

        context.getSaveOrderList().add(orderInfo);
        context.getSaveOrderSkuList().addAll(orderSkuList);
        context.getSaveOrderSkuBomList().addAll(orderSkuBomList);
        context.getOrderPackageBomList().addAll(orderPackageBomList);
    }

    /**
     * 创建订单sku
     */
    private OrderSku createOrderSku(OrderContext context, JSONObject wlnOrderSku, OrderInfo orderInfo, SkuSpec skuSpec) {
        OrderSku orderSku = new OrderSku();
        orderSku.setId(IdWorker.getId());
        orderSku.setWlnSkuId(wlnOrderSku.getString("sys_goods_uid"));
        orderSku.setWlnSkuSpecId(wlnOrderSku.getString("sys_spec_uid"));
        orderSku.setWlnSkuName(wlnOrderSku.getString("oln_sku_name"));
        orderSku.setOrderId(orderInfo.getId());
        orderSku.setQuantity(wlnOrderSku.getBigDecimal("size"));
        orderSku.setPrintType(1);
        orderSku.setStockPreparationStatus(StatusConstant.NO);

        orderSku.setUnitPrice(BigDecimal.ZERO);
        orderSku.setCustomProcessingFee(BigDecimal.ZERO);
        orderSku.setLssueFee(BigDecimal.ZERO);
        orderSku.setDeliveryMaterialsFee(BigDecimal.ZERO);
        orderSku.setPackingLabor(BigDecimal.ZERO);
        orderSku.setManagementFee(BigDecimal.ZERO);
        orderSku.setPackagingMaterialCost(BigDecimal.ZERO);

        if (skuSpec != null) {
            orderSku.setSkuId(skuSpec.getSkuId());
            orderSku.setSkuSpecId(skuSpec.getId());
            orderSku.setBomSpecId(skuSpec.getBomSpecId());
            orderSku.setBlueprint(skuSpec.getDesignImgUrl());
            orderSku.setProductionDocument(skuSpec.getSharedFolder());
            assignedOrderSkuPrice(context, orderSku, skuSpec);

            if (OrderStatusEnum.STOCK_PREPARATION.getKey().equals(orderInfo.getStatus())
                    && (StrUtil.isBlank(skuSpec.getDesignImgUrl()) || StrUtil.isBlank(skuSpec.getSharedFolder()))) {
                addExceptionType(orderInfo, OrderExceptionTypeEnum.NO_DESIGN_DOCUMENT);
            }

        }

        return orderSku;
    }

    /**
     * 赋值订单sku价格
     */
    private void assignedOrderSkuPrice(OrderContext context, OrderSku orderSku, SkuSpec skuSpec) {

        BomSpecBo bomSpecBo = context.getBomSpecBoMap().get(skuSpec.getBomSpecId());

        // 订单明细sku没绑定bom，不赋值价格
        if (bomSpecBo == null) {
            return;
        }

        // 设置单价
        orderSku.setUnitPrice(bomSpecBo.getInternalSellingPrice());

        // 如果bom规格不为主材，不赋值价格
        if (ObjectUtil.notEqual(bomSpecBo.getClassifyParentId(), 1L)) {
            return;
        }

        // 获取报价规则列表
        List<PriceBillingStandardDetail> priceBillingStandardDetailList =
                context.getPriceBillingStandardMap(bomSpecBo.getBomSpecies());

        // 报价规则列表
        if (ObjectUtil.isEmpty(priceBillingStandardDetailList)) {
            return;
        }

        for (PriceBillingStandardDetail priceBillingStandardDetail : priceBillingStandardDetailList) {
            String chargeItem = priceBillingStandardDetail.getChargeItem();
            BigDecimal chargePrice = priceBillingStandardDetail.getChargePrice();

            switch (chargeItem) {
                case "40":
                    orderSku.setPackingLabor(chargePrice);
                    break;
                case "50":
                    orderSku.setLssueFee(chargePrice);
                    break;
                case "60":
                    orderSku.setDeliveryMaterialsFee(chargePrice);
                    break;
                case "70":
                    orderSku.setManagementFee(chargePrice);
                    break;
                default:
                    if (Objects.equals(chargeItem, skuSpec.getMachinedPanel())
                            && orderSku.getQuantity().compareTo(priceBillingStandardDetail.getSectionMin()) >= 0
                            && orderSku.getQuantity().compareTo(priceBillingStandardDetail.getSectionMax()) < 0) {
                        orderSku.setCustomProcessingType(chargeItem);
                        orderSku.setCustomProcessingFee(chargePrice);
                    }
            }
        }
    }

    /**
     * 添加订单价格
     */
    private void addOrderInfoPrice(OrderInfo orderInfo, OrderSku orderSku) {
        BigDecimal quantity = orderSku.getQuantity();

        BigDecimal productTotalAmount = quantity.multiply(orderSku.getUnitPrice());
        BigDecimal customProcessingFee = quantity.multiply(orderSku.getCustomProcessingFee());
        BigDecimal lssueFee = quantity.multiply(orderSku.getLssueFee());
        BigDecimal deliveryMaterialsFee = quantity.multiply(orderSku.getDeliveryMaterialsFee());
        BigDecimal packingLabor = quantity.multiply(orderSku.getPackingLabor());
        BigDecimal packagingMaterialCost = quantity.multiply(orderSku.getPackagingMaterialCost());
        BigDecimal managementFee = quantity.multiply(orderSku.getManagementFee());
        BigDecimal totalAmount = productTotalAmount
                .add(customProcessingFee)
                .add(lssueFee)
                .add(deliveryMaterialsFee)
                .add(packingLabor)
                .add(packagingMaterialCost)
                .add(managementFee);

        orderInfo.setProductTotalAmount(orderInfo.getProductTotalAmount().add(productTotalAmount));
        orderInfo.setCustomProcessingFee(orderInfo.getCustomProcessingFee().add(customProcessingFee));
        orderInfo.setLssueFee(orderInfo.getLssueFee().add(lssueFee));
        orderInfo.setDeliveryMaterialsFee(orderInfo.getDeliveryMaterialsFee().add(deliveryMaterialsFee));
        orderInfo.setPackingLabor(orderInfo.getPackingLabor().add(packingLabor));
        orderInfo.setManagementFee(orderInfo.getManagementFee().add(managementFee));
        orderInfo.setPackagingMaterialCost(orderInfo.getPackagingMaterialCost().add(packagingMaterialCost));
        orderInfo.setTotalAmount(orderInfo.getTotalAmount().add(totalAmount));
    }

    /**
     * 添加订单异常类型
     */
    private void addExceptionType(OrderInfo orderInfo, OrderExceptionTypeEnum exceptionTypeEnum) {
        String exceptionType = orderInfo.getExceptionType();
        orderInfo.setExceptionType(orderService.addExceptionType(exceptionType, exceptionTypeEnum.getKey()));
    }

    /**
     * 保存或更新数据
     */
    @Override
    public void saveOrUpdateOrder(OrderContext context) {

        List<OrderInfo> saveOrderList = context.getSaveOrderList();
        List<OrderInfo> updateOrderList = context.getUpdateOrderList();
        List<OrderSku> saveOrderSkuList = context.getSaveOrderSkuList();
        List<OrderSkuBom> saveOrderSkuBomList = context.getSaveOrderSkuBomList();
        List<OrderPackageBom> orderPackageBomList = context.getOrderPackageBomList();

        if (saveOrderList.size() > 0) {
            orderService.saveBatch(saveOrderList);
        }
        if (updateOrderList.size() > 0) {
            orderService.updateBatchById(updateOrderList);
        }
        if (saveOrderSkuList.size() > 0) {
            orderSkuService.saveBatch(saveOrderSkuList);
        }
        if (saveOrderSkuBomList.size() > 0) {
            orderSkuBomService.saveBatch(saveOrderSkuBomList);
        }
        if (orderPackageBomList.size() > 0) {
            orderPackageBomService.saveBatch(orderPackageBomList);
        }

    }

    /**
     * 查询万里牛订单列表
     */
    private List<JSONObject> getWlnOrderList(String warehouseCode) {
        long endTime = new Date().getTime();
        long startTime;

        OrderInfo orderInfo = orderService.getOne(q -> q
                .eq(OrderInfo::getWlnStorageCode, warehouseCode)
                .orderByDesc(OrderInfo::getWlnModifyTimestamp));

        List<JSONObject> list = new ArrayList<>();

        if (orderInfo == null) {
            int day = 7;
            startTime = endTime - 1000 * 60 * 60 * 24 * day;

            for (int i = 0; i < day - 1; i++) {
                getOrderList(startTime, startTime + 1000 * 60 * 60 * 24, warehouseCode, list);
                startTime += 1000 * 60 * 60 * 24;
            }

        } else {
            startTime = orderInfo.getWlnModifyTimestamp();
            if (endTime - startTime > 1000 * 60 * 60 * 24 * 7) {
                startTime = endTime - 1000 * 60 * 60 * 24 * 7;
            }
            getOrderList(startTime, endTime, warehouseCode, list);
        }

        return list;
    }

    private void getOrderList(Long startTime, Long endTime, String warehouseCode, List<JSONObject> list) {
        int page = 1;
        int size;
        do {
            try {
                List<JSONObject> itemList = WlnUtil.getOrderList(page, 200, startTime, endTime, warehouseCode);
                page++;
                size = itemList.size();
                list.addAll(itemList);
            } catch (Exception e) {
                log.error("订单同步失败", e);
                throw new ServiceException("订单同步失败");
            }
        } while (size >= 200);
    }

}
